#include "bezier.h"
#include <Imagine/Graphics.h>
using namespace Imagine;

const float delta = 1.0f / npoints;

// Q7
Bezier initBezier() {
    Bezier B;
    B.P[0].x=0; B.P[0].y=0;
    B.P[3].x=1; B.P[3].y=0;
    B.P[1] = (B.P[0]+B.P[3])/3;
    B.P[2] = (B.P[0]+B.P[3])*2/3;
    return B;
}

// Convenience functions. Rounding int(.+.5f) prevents visual glitches for
// 4 control points with same integer x or same integer y.
void drawLine(point p, point q, Color c) {
    drawLine(int(p.x+.5f), int(p.y+.5f), int(q.x+.5f), int(q.y+.5f), c);
}
void fillCircle(point p, Color c) {
    fillCircle(int(p.x+.5f), int(p.y+.5f), radius, c);
}

void draw(Bezier B) {
    // Q9,Q10,Q12: affine transform on control pts commutes with formula P(s)
    point P[4];
    for(int i=0; i<4; i++)
        P[i] = affine(B.P[i],z,shift);
    // Q12
    for(int i=0; i+1<4; i++)
        drawLine(P[i], P[i+1], Color(200,200,200));
    // Q9
    point p = P[0];
    float s=0;
    for(int i=0; i<npoints; i++, s+=delta) {
        float t = 1-s;
        point q = t*t*t*P[0] + 3*t*t*s*P[1] + 3*t*s*s*P[2] + s*s*s*P[3];
        drawLine(p, q, BLUE);
        p = q;
    }
    // Q10
    fillCircle(P[0], RED);
    fillCircle(P[3], RED);
    fillCircle(P[1], GREEN);
    fillCircle(P[2], GREEN);
}
