// Project: Bezier (play with Bezier cubic curve)
// Author: Pascal Monasse

#include "bezier.h"
#include <Imagine/Graphics.h>
#include <iostream>
using namespace Imagine;
using namespace std;

// Q13. Turns of the control points around their closest extremity.
void animate() {
    cout << "Animation with moving control points" << endl;
    for(int i=0; i<100; i++) {
        Bezier B = initBezier();
        B.P[1].x = 0.75;
        B.P[1] = rotate(B.P[1], B.P[0], 10*i);
        B.P[2] = rotate(B.P[2], B.P[3], 3*i);
        clearWindow();
        draw(B);
        milliSleep(50);
    }
}

// Q14. Let user click on a control point of a curve. Return in x0,y0 the  click
// position and in i the point number (1 or 2). Return false on right click.
bool selectPoint(Bezier B, int& x0, int& y0, int& i) {
    i = 0;
    do {
        if(getMouse(x0,y0)==3)
            return false;
        for(int j=1; j<=2; j++) {
            point p = {(float)x0, (float)y0};
            point d = p - affine(B.P[j],z,shift);
            if(d.x*d.x+d.y*d.y <= radius*radius)
                i=j;
        }
    } while(i==0);
    return true;
}

// Return true if mouse pointer moves and put in x and y the coordinates.
// If the mouse button is released, return false.
bool track(int& x, int& y) {
    Event e;
    while(true) {
        getEvent(-1,e);
        if(e.type == EVT_BUT_OFF)
            return false;
        if(e.type == EVT_MOTION) {
            x = e.pix[0];
            y = e.pix[1];
            return true;
        }
    }
}

// Q15. Let user move freely the control points.
void interactive() {
    cout << "Interactive: select a green control point and drag it. "
         << "Right-click to finish" << endl;
    Bezier B = initBezier();
    clearWindow();
    draw(B);
    int x0, y0;
    int i=0;
    while(selectPoint(B, x0, y0, i)) {
        int x, y;
        while(track(x,y)) {
            B.P[i].x += (x-x0)/z;
            B.P[i].y += (y-y0)/z;
            x0 = x;
            y0 = y;
            clearWindow();
            draw(B);
        }
    }
}

int main() {
    // Q11
    openWindow(dim, dim, "Bezier");
    Bezier B = initBezier();
    draw(B);
    click();
    // Q13
    animate();
    click();
    // Q15
    interactive();
    endGraphics();
    return 0;
}
